

#include <linux/module.h>
#include <linux/kernel.h>

#include "setjmp.h"

static pj_jmp_buf buf;

void second(void)
{
    /* Normally the code here would try to do something useful and
	longjmp on failure, but here we're just demonstrating
	longjmp's ability to return from nested function calls.
	Note that this is not as general as a throw in other languages
	since it needs to be aware of the jmp_buf to be used to jump
	back to a specific caller.
	Alternatively the jmp_buf could be in main scope and passed in
	the function parameters, making error handling more general
	but complicating function calls. */
	pj_longjmp(buf, 3);
}

void first(void)
{
	printk("calling second\n");
    /* If second calls longjmp, this code has no way of knowing,
	so it can't perform any cleanup. */
	second();
	printk("second succeeded\n");  /* not reached */
}

int init_module(void)
{
	int specific_status = 0;

	printk(KERN_INFO "JMP test module loaded. Starting tests...\n");

	printk("1: \n");

	switch (pj_setjmp(buf)) {
		case 0: {

			/* Run code that may signal failure via longjmp. */
			printk("calling first\n");
			first();
			printk("first succeeded\n");  /* not reached */
			break;
		}
		case 3:
			printk("2: \n");

			specific_status = 3;
			/* fallthrough */
			default: {
				/* Take any action required on failure. */
				if (specific_status != 0)
					printk("first failed, status %d\n", specific_status);
				else
					printk("first failed, status unknown\n");
				break;
			}
	}

	/* Prevent module from loading. We've finished test anyway.. */
	return -1;
}

void cleanup_module(void)
{
	printk(KERN_INFO "JMP test module unloading...\n");
}

MODULE_LICENSE("GPL");

